{
--------------------------------------------------------------
-- FeatureTweezer module: Parser for feature composition order
-- 2010-10 Wolfgang Scholz
--------------------------------------------------------------
-- Tiny Parser for extracting a product line's feature composition order
--
module CompOrder where
import Char


}

%name extractCompositionOrder
-- [Token] -> [String]
-- Result will be
-- a list of feature names in the order of composition (first one first)
%tokentype { Token }
%error { parseError }

%token
    var             { TokenElem $$ }

--%left '('
--%left not
%%

CompLines :: { [String] }
    : var CompLines { ($1:$2) }
    | var           { [$1] }

{

data Token
      = TokenElem String

instance Show Token where
    show (TokenElem str) = str

parseError :: [Token] -> a
parseError [] = error "composition order parse error"
parseError (x:xs) = error ("Parse error before : \"" 
    ++ (foldr (++) "" (map (' ':) (map show (take 3 (x:xs))))) 
    ++ "\"")

-- lexer
-- note that '"' may not be part of any reserved word
lexer :: String -> [Token]
lexer [] = []
lexer (c:cs) 
      | isSpace c = lexer cs
      | otherwise = lexVar (c:cs)

lexVar cs =
   case span (\x -> not(isSpace x)) cs of 
      (var,rest)   -> (TokenElem var) : lexer rest

-- discards comments
removeComments :: String -> String
removeComments [] = ""
removeComments ('#':xs) = discardLine xs
    where discardLine :: String -> String --until EOL
          discardLine [] = ""
          discardLine (x:xs) | isPrint x = discardLine xs
                             | otherwise = removeComments xs
removeComments (x:xs) = x : (removeComments xs)

main = getContents >>= print . extractCompositionOrder . lexer . removeComments 

}

--happy example.y 
--happy example.y -i 



